@@ -4,74 +4,68 @@
  *	XSH command for POSIX
  */
 
-#include <fcntl.h>
 #ifdef	FD
 #include "fd.h"
-#else	/* !FD */
-#include "machine.h"
-#include <stdio.h>
-#include <string.h>
-#include <errno.h>
-
-#ifndef	NOUNISTDH
-#include <unistd.h>
-#endif
-
-#ifndef	NOSTDLIBH
-#include <stdlib.h>
-#endif
-
+#else
+#include "headers.h"
+#include "depend.h"
 #include "printf.h"
 #include "kctype.h"
-#include "pathname.h"
-#endif	/* !FD */
-
-#if	MSDOS
-#include "unixemu.h"
-#else
-#include <sys/param.h>
+#include "string.h"
+#include "malloc.h"
 #endif
 
+#include "sysemu.h"
+#include "unixemu.h"
+
+#if	!MSDOS
+#include "time.h"
+#endif
+#ifdef	DEP_ORIGSHELL
 #include "system.h"
-
-#if	!defined (MINIMUMSHELL) \
-&& (!defined (FD) || (FD >= 2 && !defined (_NOORIGSHELL)))
-
+#endif
+#if	defined (DEP_ORIGSHELL) && !defined (MINIMUMSHELL)
+#include "posixsh.h"
+#endif
 #ifdef	FD
 #include "term.h"
-extern int Xstat __P_((CONST char *, struct stat *));
-extern int Xlstat __P_((CONST char *, struct stat *));
-extern int Xaccess __P_((CONST char *, int));
-# ifndef	_NOPTY
+#endif
+#ifdef	DEP_PTY
 #include "termemu.h"
-extern VOID sendparent __P_((int, ...));
-# endif
-#else	/* !FD */
-extern int ttyio;
-extern FILE *ttyout;
-# if	MSDOS
-extern int Xstat __P_((CONST char *, struct stat *));
-# define	Xlstat		Xstat
-# else
-# define	Xstat(p, s)	((stat(p, s)) ? -1 : 0)
-# define	Xlstat(p, s)	((lstat(p, s)) ? -1 : 0)
-# endif
-#define	Xaccess(p, m)	((access(p, m)) ? -1 : 0)
-#endif	/* !FD */
-
-#ifndef	EPERM
-#define	EPERM		EACCES
 #endif
 
-extern VOID error __P_((CONST char *));
-extern char *malloc2 __P_((ALLOC_T));
-extern char *realloc2 __P_((VOID_P, ALLOC_T));
-extern char *strdup2 __P_((CONST char *));
-extern char *strndup2 __P_((CONST char *, int));
-extern char *strchr2 __P_((CONST char *, int));
-extern char *strncpy2 __P_((char *, CONST char *, int));
+#define	ALIASDELIMIT		"();&|<>"
+#define	getconstvar(s)		(getshellvar(s, strsize(s)))
+#define	ER_COMNOFOUND		1
+#define	ER_NOTFOUND		2
+#define	ER_NOTIDENT		4
+#define	ER_BADOPTIONS		13
+#define	ER_NUMOUTRANGE		18
+#define	ER_NOTALIAS		20
+#define	ER_MISSARG		21
+#define	ER_NOSUCHJOB		22
+#define	ER_TERMINATED		23
+#define	ER_UNKNOWNSIG		26
+
+#if	!MSDOS
+typedef struct _mailpath_t {
+	char *path;
+	char *msg;
+	time_t mtime;
+} mailpath_t;
+#endif
+
+#if	defined (DEP_ORIGSHELL) && !defined (MINIMUMSHELL)
+
 extern int setenv2 __P_((CONST char *, CONST char *, int));
-extern time_t time2 __P_((VOID_A));
+#ifdef	DEP_PTY
+extern VOID sendparent __P_((int, ...));
+#endif
+
+#ifndef	FD
+extern int ttyio;
+extern XFILE *ttyout;
+#endif
 
 #ifndef	NOJOB
 static CONST char *NEAR headstree __P_((syntaxtree *));
@@ -90,65 +84,6 @@ static int NEAR testsub1 __P_((int, CONST char *, int *));
 static int NEAR testsub2 __P_((int, char *CONST *, int *));
 static int NEAR testsub3 __P_((int, char *CONST *, int *, int));
 
-#define	ALIASDELIMIT	"();&|<>"
-#define	BUFUNIT		32
-#define	getconstvar(s)	(getshellvar(s, strsize(s)))
-#define	ER_COMNOFOUND	1
-#define	ER_NOTFOUND	2
-#define	ER_NOTIDENT	4
-#define	ER_BADOPTIONS	13
-#define	ER_NUMOUTRANGE	18
-#define	ER_NOTALIAS	20
-#define	ER_MISSARG	21
-#define	ER_NOSUCHJOB	22
-#define	ER_TERMINATED	23
-#define	ER_UNKNOWNSIG	26
-
-#if	!MSDOS
-typedef struct _mailpath_t {
-	char *path;
-	char *msg;
-	time_t mtime;
-} mailpath_t;
-#endif
-
-#ifndef	NOJOB
-int gettermio __P_((p_id_t, int));
-VOID dispjob __P_((int, FILE *));
-int searchjob __P_((p_id_t, int *));
-int getjob __P_((CONST char *));
-int stackjob __P_((p_id_t, int, syntaxtree *));
-int stoppedjob __P_((p_id_t));
-VOID killjob __P_((VOID_A));
-VOID checkjob __P_((FILE *));
-#endif	/* !NOJOB */
-char *evalposixsubst __P_((CONST char *, int *));
-#if	!MSDOS
-VOID replacemailpath __P_((CONST char *, int));
-VOID checkmail __P_((int));
-#endif
-#ifndef	NOALIAS
-int addalias __P_((char *, char *));
-int deletealias __P_((CONST char *));
-VOID freealias __P_((shaliastable *));
-int checkalias __P_((syntaxtree *, char *, int, int));
-#endif
-#ifndef	NOJOB
-int posixjobs __P_((syntaxtree *));
-int posixfg __P_((syntaxtree *));
-int posixbg __P_((syntaxtree *));
-#endif
-#ifndef	NOALIAS
-int posixalias __P_((syntaxtree *));
-int posixunalias __P_((syntaxtree *));
-#endif
-int posixkill __P_((syntaxtree *));
-int posixtest __P_((syntaxtree *));
-#ifndef	NOPOSIXUTIL
-int posixcommand __P_((syntaxtree *));
-int posixgetopts __P_((syntaxtree *));
-#endif
-
 #ifndef	NOJOB
 jobtable *joblist = NULL;
 int maxjobs = 0;
@@ -177,32 +112,31 @@ int job;
 	int ret;
 	sigmask_t mask, omask;
 
-	if (ttypgrp < (p_id_t)0 || pgrp < (p_id_t)0 || pgrp == ttypgrp)
-		return(0);
+	if (ttypgrp < (p_id_t)0 || pgrp < (p_id_t)0) return(0);
 	else if (!job) {
 		ttypgrp = pgrp;
 		return(0);
 	}
 
 	Xsigemptyset(mask);
-#ifdef	SIGTSTP
+# ifdef	SIGTSTP
 	Xsigaddset(mask, SIGTSTP);
-#endif
-#ifdef	SIGCHLD
+# endif
+# ifdef	SIGCHLD
 	Xsigaddset(mask, SIGCHLD);
-#endif
-#ifdef	SIGTTIN
+# endif
+# ifdef	SIGTTIN
 	Xsigaddset(mask, SIGTTIN);
-#endif
-#ifdef	SIGTTOU
+# endif
+# ifdef	SIGTTOU
 	Xsigaddset(mask, SIGTTOU);
-#endif
+# endif
 	Xsigblock(omask, mask);
 
-#ifdef	JOBVERBOSE
-	fprintf2(ttyout, "gettermio: %id: %id -> %id", mypid, ttypgrp, pgrp);
-	fputnl(ttyout);
-#endif	/* JOBVERBOSE */
+# ifdef	JOBVERBOSE
+	VOID_C Xfprintf(ttyout,
+		"gettermio: %id: %id -> %id\n", mypid, ttypgrp, pgrp);
+# endif
 	if ((ret = settcpgrp(ttyio, pgrp)) >= 0) ttypgrp = pgrp;
 	Xsigsetmask(omask);
 
@@ -231,44 +165,86 @@ syntaxtree *trp;
 
 VOID dispjob(n, fp)
 int n;
-FILE *fp;
+XFILE *fp;
 {
 	int i, sig;
 
 	if (n < 0 || n >= maxjobs || !(joblist[n].pids)) return;
 	i = joblist[n].npipe;
-	fprintf2(fp, "[%d]%c %id ",
+	VOID_C Xfprintf(fp,
+		"[%d]%c %id ",
 		n + 1, (n == lastjob) ? '+' : ((n == prevjob) ? '-' : ' '),
 		joblist[n].pids[i]);
 	sig = joblist[n].stats[i];
 
 	if (sig <= 0)
-		fprintf2(fp, "%-28.28s", (sig) ? "Done" : "Running");
+		VOID_C Xfprintf(fp, "%-28.28s", (sig) ? "Done" : "Running");
 	else {
 		if (sig >= 128) sig -= 128;
 		dispsignal(sig, 28, fp);
 	}
 	if (joblist[n].trp) {
+# ifdef	DEP_FILECONV
+		if (joblist[n].trp -> flags & ST_DEFKANJI) printf_defkanji++;
+# endif
 		if (!isopbg(joblist[n].trp) && !isopnown(joblist[n].trp)
 		&& !isoppipe(joblist[n].trp))
 			joblist[n].trp -> type = OP_NONE;
 		printstree(joblist[n].trp, -1, fp);
+# ifdef	DEP_FILECONV
+		if (joblist[n].trp -> flags & ST_DEFKANJI) printf_defkanji--;
+# endif
 	}
-	fputnl(fp);
+	VOID_C fputnl(fp);
+}
+
+int statjob(n)
+int n;
+{
+	int i, sig;
+
+	if (n < 0 || n >= maxjobs || !(joblist[n].pids)) return(-1);
+
+	sig = -1;
+	for (i = 0; i <= joblist[n].npipe; i++) {
+		if (joblist[n].stats[i] < 0 || joblist[n].stats[i] >= 128)
+			continue;
+		if (sig < joblist[n].stats[i]) sig = joblist[n].stats[i];
+	}
+
+	return(sig);
+}
+
+VOID freejob(n)
+int n;
+{
+	if (n < 0 || n >= maxjobs) return;
+	if (!(joblist[n].pids)) return;
+
+	Xfree(joblist[n].pids);
+	Xfree(joblist[n].stats);
+	Xfree(joblist[n].fds);
+	if (joblist[n].trp) {
+		freestree(joblist[n].trp);
+		Xfree(joblist[n].trp);
+	}
+	Xfree(joblist[n].tty);
+
+	joblist[n].pids = NULL;
 }
 
 int searchjob(pid, np)
 p_id_t pid;
 int *np;
 {
-	int i, j;
+	int i, n;
 
-	for (i = 0; i < maxjobs; i++) {
-		if (!(joblist[i].pids)) continue;
-		for (j = 0; j <= joblist[i].npipe; j++) {
-			if (joblist[i].pids[j] != pid) continue;
-			if (np) *np = j;
-			return(i);
+	for (n = 0; n < maxjobs; n++) {
+		if (!(joblist[n].pids)) continue;
+		for (i = 0; i <= joblist[n].npipe; i++) {
+			if (joblist[n].pids[i] != pid) continue;
+			if (np) *np = i;
+			return(n);
 		}
 	}
 
@@ -279,173 +255,159 @@ int getjob(s)
 CONST char *s;
 {
 	CONST char *cp;
-	int i, j;
+	int n, len;
 
 	if (!jobok) return(-1);
-	if (!s) i = lastjob;
+	if (!s) n = lastjob;
 	else {
 		if (s[0] != '%') return(-1);
 		if (!s[1] || ((s[1] == '%' || s[1] == '+') && !s[2]))
-			i = lastjob;
-		else if (s[1] == '-' && !s[2]) i = prevjob;
-		else if ((i = isnumeric(&(s[1]))) >= 0) i--;
+			n = lastjob;
+		else if (s[1] == '-' && !s[2]) n = prevjob;
+		else if ((n = isnumeric(&(s[1]))) >= 0) n--;
 		else {
-			j = strlen(&(s[1]));
-			for (i = 0; i < maxjobs; i++) {
-				if (!(joblist[i].pids) || !(joblist[i].trp)
-				|| !(cp = headstree(joblist[i].trp)))
+			len = strlen(&(s[1]));
+			for (n = 0; n < maxjobs; n++) {
+				if (!(joblist[n].pids) || !(joblist[n].trp)
+				|| !(cp = headstree(joblist[n].trp)))
 					continue;
-				if (!strnpathcmp(&(s[1]), cp, j)) break;
+				if (!strnpathcmp(&(s[1]), cp, len)) break;
 			}
 		}
 	}
-	if (i < 0 || i >= maxjobs || !joblist || !(joblist[i].pids))
+	if (n < 0 || n >= maxjobs || !joblist || !(joblist[n].pids))
 		return(-1);
-	j = joblist[i].npipe;
-	if (joblist[i].stats[j] < 0 || joblist[i].stats[j] >= 128) return(-2);
+	if (statjob(n) < 0) return(-2);
 
-	return(i);
+	return(n);
 }
 
-int stackjob(pid, sig, trp)
+int stackjob(pid, sig, fd, trp)
 p_id_t pid;
-int sig;
+int sig, fd;
 syntaxtree *trp;
 {
-	int i, j, n;
+	int i, n, min;
 
 #ifdef	FAKEUNINIT
-	j = 0;	/* fake for -Wuninitialized */
+	i = 0;	/* fake for -Wuninitialized */
 #endif
 	if (!joblist) {
-		joblist = (jobtable *)malloc2(BUFUNIT * sizeof(jobtable));
+		joblist = (jobtable *)Xmalloc(BUFUNIT * sizeof(jobtable));
 		maxjobs = BUFUNIT;
-		i = 0;
-		for (n = 0; n < maxjobs; n++) joblist[n].pids = NULL;
+		n = 0;
+		for (i = 0; i < maxjobs; i++) joblist[i].pids = NULL;
 	}
 	else {
-		n = -1;
-		for (i = 0; i < maxjobs; i++) {
-			if (!(joblist[i].pids)) {
-				if (n < 0) n = i;
+		min = -1;
+		for (n = 0; n < maxjobs; n++) {
+			if (!(joblist[n].pids)) {
+				if (min < 0) min = n;
 				continue;
 			}
-			for (j = 0; j <= joblist[i].npipe; j++)
-				if (joblist[i].pids[j] == pid) break;
-			if (j <= joblist[i].npipe) break;
-			else if (joblist[i].pids[0] == childpgrp) {
-				j = joblist[i].npipe + 1;
+			for (i = 0; i <= joblist[n].npipe; i++)
+				if (joblist[n].pids[i] == pid) break;
+			if (i <= joblist[n].npipe) break;
+			else if (joblist[n].pids[0] == childpgrp) {
+				i = joblist[n].npipe + 1;
 				break;
 			}
 		}
-		if (i < maxjobs) /*EMPTY*/;
-		else if (n >= 0) i = n;
+		if (n < maxjobs) /*EMPTY*/;
+		else if (min >= 0) n = min;
 		else {
-			joblist = (jobtable *)realloc2(joblist,
+			joblist = (jobtable *)Xrealloc(joblist,
 				(maxjobs + BUFUNIT) * sizeof(jobtable));
 			maxjobs += BUFUNIT;
-			for (n = i; n < maxjobs; n++) joblist[n].pids = NULL;
+			for (i = n; i < maxjobs; i++) joblist[i].pids = NULL;
 		}
 	}
 
-	if (!(joblist[i].pids)) {
-		joblist[i].pids = (p_id_t *)malloc2(BUFUNIT * sizeof(p_id_t));
-		joblist[i].stats = (int *)malloc2(BUFUNIT * sizeof(int));
-		joblist[i].npipe = 0;
-		joblist[i].trp = NULL;
-		joblist[i].tty = NULL;
-# ifdef	USESGTTY
-		joblist[i].ttyflag = NULL;
+	if (!(joblist[n].pids)) {
+		joblist[n].pids = (p_id_t *)Xmalloc(BUFUNIT * sizeof(p_id_t));
+		joblist[n].stats = (int *)Xmalloc(BUFUNIT * sizeof(int));
+		joblist[n].fds = (int *)Xmalloc(BUFUNIT * sizeof(int));
+		joblist[n].npipe = 0;
+		joblist[n].trp = NULL;
+		joblist[n].tty = NULL;
+		i = 0;
+	}
+	else if (i > joblist[n].npipe) {
+		if (!(i % BUFUNIT)) {
+			joblist[n].pids = (p_id_t *)Xrealloc(joblist[n].pids,
+				(i + BUFUNIT) * sizeof(p_id_t));
+			joblist[n].stats = (int *)Xrealloc(joblist[n].stats,
+				(i + BUFUNIT) * sizeof(int));
+			joblist[n].fds = (int *)Xrealloc(joblist[n].fds,
+				(i + BUFUNIT) * sizeof(int));
+		}
+		joblist[n].npipe = i;
+	}
+	else if (fd < 0) fd = joblist[n].fds[i];
+
+	joblist[n].pids[i] = pid;
+	joblist[n].stats[i] = sig;
+	joblist[n].fds[i] = fd;
+	if (!i && !(joblist[n].trp) && trp) {
+		joblist[n].trp = duplstree2(trp, NULL, 0L);
+		if (!isoppipe(joblist[n].trp) && joblist[n].trp -> next) {
+			freestree(joblist[n].trp -> next);
+			joblist[n].trp -> next = NULL;
+		}
+	}
+
+# ifdef	JOBVERBOSE
+	VOID_C Xfprintf(ttyout, "stackjob: %id: %id, %d:", mypid, pid, n);
+	for (i = 0; i <= joblist[n].npipe; i++)
+		VOID_C Xfprintf(ttyout, "%id ", joblist[n].pids[i]);
+	VOID_C fputnl(ttyout);
 # endif
-		j = 0;
-	}
-	else if (j > joblist[i].npipe) {
-		if (!(j % BUFUNIT)) {
-			joblist[i].pids = (p_id_t *)realloc2(joblist[i].pids,
-				(j + BUFUNIT) * sizeof(p_id_t));
-			joblist[i].stats = (int *)realloc2(joblist[i].stats,
-				(j + BUFUNIT) * sizeof(int));
-		}
-		joblist[i].npipe = j;
-	}
 
-	joblist[i].pids[j] = pid;
-	joblist[i].stats[j] = sig;
-	if (!j && !(joblist[i].trp) && trp) {
-		joblist[i].trp = duplstree(trp, NULL, 0L);
-		if (!isoppipe(joblist[i].trp) && joblist[i].trp -> next) {
-			freestree(joblist[i].trp -> next);
-			joblist[i].trp -> next = NULL;
-		}
-	}
-
-#ifdef	JOBVERBOSE
-	fprintf2(ttyout, "stackjob: %id: %id, %d:", mypid, pid, i);
-	for (j = 0; j <= joblist[i].npipe; j++)
-		fprintf2(ttyout, "%id ", joblist[i].pids[j]);
-	fputnl(ttyout);
-#endif	/* JOBVERBOSE */
-
-	return(i);
+	return(n);
 }
 
 int stoppedjob(pid)
 p_id_t pid;
 {
-	int i, j, sig;
+	int i, n;
 
 	if (stopped) return(1);
-	if ((i = searchjob(pid, &j)) >= 0) {
-		for (; j <= joblist[i].npipe; j++) {
-			sig = joblist[i].stats[j];
-			if (sig > 0 && sig < 128) return(1);
-			else if (!sig) return(0);
-		}
+	if ((n = searchjob(pid, &i)) < 0) /*EMPTY*/;
+	else for (; i <= joblist[n].npipe; i++) {
+		if (joblist[n].stats[i] < 0 || joblist[n].stats[i] >= 128)
+			continue;
+		return((joblist[n].stats[i]) ? 1 : 0);
 	}
 
 	return(-1);
 }
 
 VOID checkjob(fp)
-FILE *fp;
+XFILE *fp;
 {
-	int i, j;
+	int n;
 
-	while (waitjob(-1, NULL, WNOHANG | WUNTRACED) > 0);
-	if (fp) for (i = 0; i < maxjobs; i++) {
-		if (!(joblist[i].pids)) continue;
-		j = joblist[i].npipe;
-		if (joblist[i].stats[j] >= 0 && joblist[i].stats[j] < 128)
-			continue;
+	while (waitjob(-1, NULL, WNOHANG | WUNTRACED) > 0) /*EMPTY*/;
+	if (fp) for (n = 0; n < maxjobs; n++) {
+		if (!(joblist[n].pids) || statjob(n) >= 0) continue;
 
-		if (joblist[i].trp
-		&& (isopbg(joblist[i].trp) || isopnown(joblist[i].trp)))
-			joblist[i].trp -> type = OP_NONE;
-		if (jobok && interactive && !nottyout) dispjob(i, fp);
-		free(joblist[i].pids);
-		free(joblist[i].stats);
-		if (joblist[i].trp) {
-			freestree(joblist[i].trp);
-			free(joblist[i].trp);
-		}
-		if (joblist[i].tty) free(joblist[i].tty);
-# ifdef	USESGTTY
-		if (joblist[i].ttyflag) free(joblist[i].ttyflag);
-# endif
-		joblist[i].pids = NULL;
+		if (joblist[n].trp
+		&& (isopbg(joblist[n].trp) || isopnown(joblist[n].trp)))
+			joblist[n].trp -> type = OP_NONE;
+		if (jobok && interactive && !nottyout) dispjob(n, fp);
+		freejob(n);
 	}
 }
 
 VOID killjob(VOID_A)
 {
-	int i, j;
+	int n, sig;
 
-	for (i = 0; i < maxjobs; i++) {
-		if (!(joblist[i].pids)) continue;
-		j = joblist[i].stats[joblist[i].npipe];
-		if (j < 0 || j >= 128) continue;
-		Xkillpg(joblist[i].pids[0], SIGHUP);
-		if (j > 0) Xkillpg(joblist[i].pids[0], SIGCONT);
+	for (n = 0; n < maxjobs; n++) {
+		if (!(joblist[n].pids) || (sig = statjob(n)) < 0) continue;
+
+		Xkillpg(joblist[n].pids[0], SIGHUP);
+		if (sig > 0) Xkillpg(joblist[n].pids[0], SIGCONT);
 	}
 }
 #endif	/* !NOJOB */
@@ -473,18 +435,18 @@ char *ident, *comm;
 
 	i = searchalias(ident, -1);
 	if (shellalias[i].ident) {
-		free(shellalias[i].ident);
-		free(shellalias[i].comm);
+		Xfree(shellalias[i].ident);
+		Xfree(shellalias[i].comm);
 	}
 	else {
-		shellalias = (shaliastable *)realloc2(shellalias,
+		shellalias = (shaliastable *)Xrealloc(shellalias,
 			(i + 2) * sizeof(shaliastable));
 		shellalias[i + 1].ident = NULL;
 	}
 
 	shellalias[i].ident = ident;
 	shellalias[i].comm = comm;
-# if	defined (FD) && !defined (_NOPTY)
+# ifdef	DEP_PTY
 	sendparent(TE_ADDALIAS, ident, comm);
 # endif
 
@@ -494,27 +456,27 @@ char *ident, *comm;
 int deletealias(ident)
 CONST char *ident;
 {
-	reg_t *re;
+	reg_ex_t *re;
 	int i, n, max;
 
 	n = 0;
 	re = regexp_init(ident, -1);
-	for (max = 0; shellalias[max].ident; max++);
+	for (max = 0; shellalias[max].ident; max++) /*EMPTY*/;
 	for (i = 0; i < max; i++) {
 		if (re) {
 			if (!regexp_exec(re, shellalias[i].ident, 0)) continue;
 		}
 		else if (strcommcmp(ident, shellalias[i].ident)) continue;
-		free(shellalias[i].ident);
-		free(shellalias[i].comm);
+		Xfree(shellalias[i].ident);
+		Xfree(shellalias[i].comm);
 		memmove((char *)&(shellalias[i]), (char *)&(shellalias[i + 1]),
 			(max-- - i) * sizeof(shaliastable));
 		i--;
 		n++;
 	}
-	if (re) regexp_free(re);
+	regexp_free(re);
 	if (!n) return(-1);
-# if	defined (FD) && !defined (_NOPTY)
+# ifdef	DEP_PTY
 	sendparent(TE_DELETEALIAS, ident);
 # endif
 
@@ -528,11 +490,11 @@ shaliastable *alias;
 	int i, n;
 
 	if (!alias) n = 0;
-	else for (n = 0; alias[n].ident; n++);
-	dupl = (shaliastable *)malloc2((n + 1) * sizeof(shaliastable));
+	else for (n = 0; alias[n].ident; n++) /*EMPTY*/;
+	dupl = (shaliastable *)Xmalloc((n + 1) * sizeof(shaliastable));
 	for (i = 0; i < n; i++) {
-		dupl[i].ident = strdup2(alias[i].ident);
-		dupl[i].comm = strdup2(alias[i].comm);
+		dupl[i].ident = Xstrdup(alias[i].ident);
+		dupl[i].comm = Xstrdup(alias[i].comm);
 	}
 	dupl[i].ident = NULL;
 
@@ -546,10 +508,10 @@ shaliastable *alias;
 
 	if (alias) {
 		for (i = 0; alias[i].ident; i++) {
-			free(alias[i].ident);
-			free(alias[i].comm);
+			Xfree(alias[i].ident);
+			Xfree(alias[i].comm);
 		}
-		free(alias);
+		Xfree(alias);
 	}
 }
 
@@ -562,7 +524,7 @@ CONST VOID_P vp2;
 	ap1 = (shaliastable *)vp1;
 	ap2 = (shaliastable *)vp2;
 
-	return(strpathcmp2(ap1 -> ident, ap2 -> ident));
+	return(strverscmp2(ap1 -> ident, ap2 -> ident));
 }
 
 int checkalias(trp, ident, len, delim)
@@ -578,7 +540,8 @@ int len, delim;
 	&& !(statementlist[i].type & STT_NEEDLIST)))
 		return(-1);
 
-	if ((!strchr(IFS_SET, delim) && !strchr(ALIASDELIMIT, delim)))
+	if (delim
+	&& (!Xstrchr(IFS_SET, delim) && !Xstrchr(ALIASDELIMIT, delim)))
 		return(-1);
 	i = searchalias(ident, len);
 	if (!(shellalias[i].ident) || (shellalias[i].flags & AL_USED))
@@ -599,7 +562,7 @@ long *resultp;
 	long n;
 	int i, c, base;
 
-	while (s[ptr] && strchr(IFS_SET, s[ptr])) ptr++;
+	while (s[ptr] && Xstrchr(IFS_SET, s[ptr])) ptr++;
 	if (!s[ptr]) return(-1);
 
 	if (s[ptr] == '+') {
@@ -624,12 +587,12 @@ long *resultp;
 	else if (s[ptr] == '(') {
 		if ((ptr = evalexpression(s, ptr + 1, resultp, 9)) < 0)
 			return(-1);
-		while (strchr(IFS_SET, s[ptr])) ptr++;
+		while (s[ptr] && Xstrchr(IFS_SET, s[ptr])) ptr++;
 		if (s[ptr++] != ')') return(-1);
 	}
-	else if (isdigit2(s[ptr])) {
+	else if (Xisdigit(s[ptr])) {
 		if (s[ptr] != '0') base = 10;
-		else if (toupper2(s[++ptr]) != 'X') base = 8;
+		else if (Xtoupper(s[++ptr]) != 'X') base = 8;
 		else {
 			base = 16;
 			ptr++;
@@ -638,9 +601,9 @@ long *resultp;
 		n = 0;
 		for (i = ptr; s[i]; i++) {
 			c = s[i];
-			if (isdigit2(c)) c -= '0';
-			else if (islower2(c)) c -= 'a' - 10;
-			else if (isupper2(c)) c -= 'A' - 10;
+			if (Xisdigit(c)) c -= '0';
+			else if (Xislower(c)) c -= 'a' - 10;
+			else if (Xisupper(c)) c -= 'A' - 10;
 			else c = -1;
 			if (c < 0 || c >= base) break;
 
@@ -656,7 +619,7 @@ long *resultp;
 	}
 	else return(-1);
 
-	while (s[ptr] && strchr(IFS_SET, s[ptr])) ptr++;
+	while (s[ptr] && Xstrchr(IFS_SET, s[ptr])) ptr++;
 
 	return(ptr);
 }
@@ -809,6 +772,10 @@ char *evalposixsubst(s, ptrp)
 CONST char *s;
 int *ptrp;
 {
+#ifndef	BASHBUG
+	syntaxtree *trp, *stree;
+	redirectlist red;
+#endif
 	char *cp, *new;
 	long n;
 	int i, len;
@@ -818,15 +785,12 @@ int *ptrp;
 
 	s += i;
 	len = *ptrp - i * 2;
-	s = new = strndup2(s, len);
+	s = new = Xstrndup(s, len);
 	if (i <= 1) {
 #ifndef	BASHBUG
 	/* bash cannot include 'case' statement within $() */
-		syntaxtree *trp, *stree;
-		redirectlist red;
-
 		red.fd = red.type = red.new = 0;
-		red.filename = (char *)nullstr;
+		red.filename = vnullstr;
 		i = len = 0;
 		stree = newstree(NULL);
 		trp = startvar(stree, &red, nullstr, &i, &len, 0);
@@ -834,7 +798,7 @@ int *ptrp;
 		trp = analyze(s, trp, -1);
 		if (trp && (trp -> cont & CN_SBST) != CN_CASE) trp = NULL;
 		freestree(stree);
-		free(stree);
+		Xfree(stree);
 		if (trp) cp = NULL;
 		else
 #endif	/* !BASHBUG */
@@ -842,20 +806,23 @@ int *ptrp;
 	}
 	else {
 		n = 0L;
-		if (!(cp = evalvararg(new, '\'', EA_BACKQ, 0))) *ptrp = -1;
+		cp = evalvararg(new,
+			EA_STRIPESCAPE
+			| EA_NOEVALQ | EA_NOEVALDQ | EA_BACKQ, 0);
+		if (!cp) *ptrp = -1;
 		else {
-			for (i = 0; cp[i] && strchr(IFS_SET, cp[i]); i++);
+			for (i = 0; cp[i]; i++)
+				if (!Xstrchr(IFS_SET, cp[i])) break;
 			if (cp[i]) {
 				i = evalexpression(cp, i, &n, 9);
 				if (i < 0 || cp[i]) *ptrp = -1;
 			}
-			free(cp);
+			Xfree(cp);
 		}
 
-		if (*ptrp < 0) cp = NULL;
-		else if (asprintf2(&cp, "%ld", n) < 0) error("malloc()");
+		cp = (*ptrp < 0) ? NULL : asprintf2("%ld", n);
 	}
-	free(new);
+	Xfree(new);
 
 	return(cp);
 }
@@ -870,14 +837,12 @@ time_t mtime;
 
 	for (i = 0; i < maxmailpath; i++)
 		if (!strpathcmp(path, mailpathlist[i].path)) break;
-	if (i < maxmailpath) {
-		if (mailpathlist[i].msg) free(mailpathlist[i].msg);
-	}
+	if (i < maxmailpath) Xfree(mailpathlist[i].msg);
 	else {
-		mailpathlist = (mailpath_t *)realloc2(mailpathlist,
+		mailpathlist = (mailpath_t *)Xrealloc(mailpathlist,
 			(i + 1) * sizeof(mailpath_t));
 		maxmailpath = i + 1;
-		mailpathlist[i].path = strdup2(path);
+		mailpathlist[i].path = Xstrdup(path);
 	}
 	mailpathlist[i].msg = msg;
 	mailpathlist[i].mtime = mtime;
@@ -905,20 +870,20 @@ int multi;
 		addmailpath(s, NULL, mtime);
 	}
 	else while (*s) {
-		if ((cp = strchr(s, ':'))) len = (cp++) - s;
+		if ((cp = Xstrchr(s, ':'))) len = (cp++) - s;
 		else {
 			len = strlen(s);
 			cp = &(s[len]);
 		}
-		if ((msg = strchr(s, '%'))) {
+		if ((msg = Xstrchr(s, '%'))) {
 			if (msg > s + len) msg = NULL;
 			else {
 				i = s + len - (msg + 1);
 				len = msg - s;
-				msg = strndup2(&(s[len + 1]), i);
+				msg = Xstrndup(&(s[len + 1]), i);
 			}
 		}
-		strncpy2(path, s, len);
+		Xstrncpy(path, s, len);
 		s = cp;
 		for (i = 0; i < max; i++)
 			if (!strpathcmp(path, mailpath[i].path)) break;
@@ -927,10 +892,10 @@ int multi;
 	}
 
 	for (i = 0; i < max; i++) {
-		free(mailpath[i].path);
-		if (mailpath[i].msg) free(mailpath[i].msg);
+		Xfree(mailpath[i].path);
+		Xfree(mailpath[i].msg);
 	}
-	if (mailpath) free(mailpath);
+	Xfree(mailpath);
 }
 
 VOID checkmail(reset)
@@ -942,17 +907,17 @@ int reset;
 
 	if (reset) {
 		for (i = 0; i < maxmailpath; i++) {
-			free(mailpathlist[i].path);
-			if (mailpathlist[i].msg) free(mailpathlist[i].msg);
+			Xfree(mailpathlist[i].path);
+			Xfree(mailpathlist[i].msg);
 		}
-		if (mailpathlist) free(mailpathlist);
+		Xfree(mailpathlist);
 		mailpathlist = NULL;
 		maxmailpath = 0;
 		return;
 	}
 
 	if (mailcheck > 0) {
-		now = time2();
+		now = Xtime(NULL);
 		if (now < lastchecked + mailcheck) return;
 		lastchecked = now;
 	}
@@ -968,27 +933,19 @@ int reset;
 int posixjobs(trp)
 syntaxtree *trp;
 {
-	int i, j;
+	int n;
 
 	if (mypid != orgpgrp) return(RET_SUCCESS);
 	checkjob(NULL);
-	for (i = 0; i < maxjobs; i++) {
-		if (!(joblist[i].pids)) continue;
-		j = joblist[i].npipe;
+	for (n = 0; n < maxjobs; n++) {
+		if (!(joblist[n].pids)) continue;
 
-		if (joblist[i].stats[j] >= 0 && joblist[i].stats[j] < 128)
-			dispjob(i, stdout);
+		if (statjob(n) >= 0) dispjob(n, Xstdout);
 		else {
-			if (isopbg(joblist[i].trp) || isopnown(joblist[i].trp))
-				joblist[i].trp -> type = OP_NONE;
-			dispjob(i, stdout);
-			free(joblist[i].pids);
-			free(joblist[i].stats);
-			if (joblist[i].trp) {
-				freestree(joblist[i].trp);
-				free(joblist[i].trp);
-			}
-			joblist[i].pids = NULL;
+			if (isopbg(joblist[n].trp) || isopnown(joblist[n].trp))
+				joblist[n].trp -> type = OP_NONE;
+			dispjob(n, Xstdout);
+			freejob(n);
 		}
 	}
 
@@ -999,43 +956,40 @@ syntaxtree *trp;
 int posixfg(trp)
 syntaxtree *trp;
 {
-	termioctl_t tty;
-	char *s;
-	int i, j, n, ret;
+	char *s, *tty;
+	int i, n, ret;
 
 	s = ((trp -> comm) -> argc > 1) ? (trp -> comm) -> argv[1] : NULL;
 	checkjob(NULL);
-	if ((i = getjob(s)) < 0) {
-		execerror((trp -> comm) -> argv[1],
-			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 0);
+	if ((n = getjob(s)) < 0) {
+		execerror((trp -> comm) -> argv,
+			(trp -> comm) -> argv[1],
+			(n < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 2);
 		return(RET_FAIL);
 	}
-	n = joblist[i].npipe;
-	if (tioctl(ttyio, REQGETP, &tty) < 0) {
-		doperror((trp -> comm) -> argv[0], "fatal error");
-		prepareexit(-1);
-		Xexit(RET_FATALERR);
+	savetermio(ttyio, &tty, NULL);
+	VOID_C Xfprintf(Xstderr, "[%d] %id\n",
+		n + 1, joblist[n].pids[joblist[n].npipe]);
+	loadtermio(ttyio, joblist[n].tty, NULL);
+	VOID_C gettermio(joblist[n].pids[0], jobok);
+	childpgrp = joblist[n].pids[0];
+	if (statjob(n) > 0) Xkillpg(joblist[n].pids[0], SIGCONT);
+	for (i = 0; i <= joblist[n].npipe; i++) {
+		if (joblist[n].stats[i] < 0 || joblist[n].stats[i] >= 128)
+			continue;
+		joblist[n].stats[i] = 0;
 	}
-	fprintf2(stderr, "[%d] %id", i + 1, joblist[i].pids[n]);
-	fputnl(stderr);
-	if (joblist[i].tty) tioctl(ttyio, REQSETP, joblist[i].tty);
-# ifdef	USESGTTY
-	if (joblist[i].ttyflag) ioctl(ttyio, TIOCLSET, joblist[i].ttyflag);
-# endif
-	gettermio(joblist[i].pids[0], jobok);
-	if ((j = joblist[i].stats[n]) > 0 && j < 128) {
-		Xkillpg(joblist[i].pids[0], SIGCONT);
-		for (j = 0; j <= n; j++) joblist[i].stats[j] = 0;
-	}
-	if (isopbg(joblist[i].trp) || isopnown(joblist[i].trp))
-		joblist[i].trp -> type = OP_NONE;
-	ret = waitchild(joblist[i].pids[n], NULL);
-	gettermio(orgpgrp, jobok);
-	if (tioctl(ttyio, REQSETP, &tty) < 0) {
-		doperror((trp -> comm) -> argv[0], "fatal error");
-		prepareexit(-1);
-		Xexit(RET_FATALERR);
+	if (isopbg(joblist[n].trp) || isopnown(joblist[n].trp))
+		joblist[n].trp -> type = OP_NONE;
+	i = joblist[n].npipe;
+	ret = waitchild(joblist[n].pids[i], NULL);
+	while (i-- > 0) {
+		if (joblist[n].fds[i] < 0) continue;
+		VOID_C closepipe2(joblist[n].fds[i], -1);
 	}
+	VOID_C gettermio(orgpgrp, jobok);
+	loadtermio(ttyio, tty, NULL);
+	Xfree(tty);
 
 	return(ret);
 }
@@ -1045,25 +999,27 @@ int posixbg(trp)
 syntaxtree *trp;
 {
 	char *s;
-	int i, j, n;
+	int i, n;
 
 	s = ((trp -> comm) -> argc > 1) ? (trp -> comm) -> argv[1] : NULL;
 	checkjob(NULL);
-	if ((i = getjob(s)) < 0) {
-		execerror((trp -> comm) -> argv[1],
-			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 0);
+	if ((n = getjob(s)) < 0) {
+		execerror((trp -> comm) -> argv,
+			(trp -> comm) -> argv[1],
+			(n < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 2);
 		return(RET_FAIL);
 	}
-	n = joblist[i].npipe;
-	if ((j = joblist[i].stats[n]) > 0 && j < 128) {
-		Xkillpg(joblist[i].pids[0], SIGCONT);
-		for (j = 0; j <= n; j++) joblist[i].stats[j] = 0;
+	if (statjob(n) > 0) Xkillpg(joblist[n].pids[0], SIGCONT);
+	for (i = 0; i <= joblist[n].npipe; i++) {
+		if (joblist[n].stats[i] < 0 || joblist[n].stats[i] >= 128)
+			continue;
+		joblist[n].stats[i] = 0;
 	}
-	if (!isopbg(joblist[i].trp) || !isopnown(joblist[i].trp))
-		joblist[i].trp -> type = OP_BG;
-	if (interactive && !nottyout) dispjob(i, stderr);
+	if (!isopbg(joblist[n].trp) || !isopnown(joblist[n].trp))
+		joblist[n].trp -> type = OP_BG;
+	if (interactive && !nottyout) dispjob(n, Xstderr);
 	prevjob = lastjob;
-	lastjob = i;
+	lastjob = n;
 
 	return(RET_SUCCESS);
 }
@@ -1080,13 +1036,11 @@ syntaxtree *trp;
 	argv = (trp -> comm) -> argv;
 	if ((trp -> comm) -> argc <= 1) {
 		alias = duplalias(shellalias);
-		for (i = 0; alias[i].ident; i++);
+		for (i = 0; alias[i].ident; i++) /*EMPTY*/;
 		if (i > 1) qsort(alias, i, sizeof(shaliastable), cmpalias);
-		for (i = 0; alias[i].ident; i++) {
-			fprintf2(stdout, "alias %k='%k'",
+		for (i = 0; alias[i].ident; i++)
+			VOID_C Xprintf("alias %k='%k'\n",
 				alias[i].ident, alias[i].comm);
-			fputnl(stdout);
-		}
 		freealias(alias);
 		return(RET_SUCCESS);
 	}
@@ -1099,21 +1053,21 @@ syntaxtree *trp;
 			break;
 		}
 
-		if (set) VOID_C addalias(strndup2(argv[n], len),
-			strdup2(&(argv[n][len + 1])));
+		if (set) VOID_C addalias(Xstrndup(argv[n], len),
+			Xstrdup(&(argv[n][len + 1])));
 		else {
 			i = searchalias(argv[n], len);
 			if (!(shellalias[i].ident)) {
 				if (interactive) {
-					fputs("alias: ", stderr);
-					execerror(argv[n], ER_NOTFOUND, 0);
+					Xfputs("alias: ", Xstderr);
+					execerror(argv,
+						argv[n], ER_NOTFOUND, 2);
 				}
 				ret = RET_FAIL;
 				ERRBREAK;
 			}
-			fprintf2(stdout, "alias %k='%k'",
+			VOID_C Xprintf("alias %k='%k'\n",
 				shellalias[i].ident, shellalias[i].comm);
-			fputnl(stdout);
 		}
 	}
 
@@ -1131,8 +1085,8 @@ syntaxtree *trp;
 	for (n = 1; n < (trp -> comm) -> argc; n++) {
 		if (deletealias(argv[n]) < 0) {
 			if (interactive) {
-				fputs("alias: ", stderr);
-				execerror(argv[n], ER_NOTALIAS, 0);
+				Xfputs("alias: ", Xstderr);
+				execerror(argv, argv[n], ER_NOTALIAS, 2);
 			}
 			ret = RET_FAIL;
 			ERRBREAK;
@@ -1151,9 +1105,9 @@ syntaxtree *trp;
 	int i, n, sig;
 
 	if ((trp -> comm) -> argc <= 1) {
-		fputs("usage: kill [ -sig ] pid ...\n", stderr);
-		fputs("for a list of signals: kill -l", stderr);
-		fputnl(stderr);
+		Xfputs("usage: kill [ -sig ] pid ...\n", Xstderr);
+		Xfputs("for a list of signals: kill -l", Xstderr);
+		VOID_C fputnl(Xstderr);
 		return(RET_SYNTAXERR);
 	}
 #ifdef	SIGTERM
@@ -1169,12 +1123,12 @@ syntaxtree *trp;
 				for (i = 0; signallist[i].sig >= 0; i++)
 					if (sig == signallist[i].sig) break;
 				if (signallist[i].sig < 0) continue;
-				fprintf2(stdout, "%s%c",
+				VOID_C Xprintf("%s%c",
 					signallist[i].ident,
 					(++n % 16) ? ' ' : '\n');
 			}
-			if (n % 16) fputnl(stdout);
-			fflush(stdout);
+			if (n % 16) VOID_C fputnl(Xstdout);
+			Xfflush(Xstdout);
 			return(RET_SUCCESS);
 		}
 		if ((sig = isnumeric(&(s[1]))) < 0) {
@@ -1182,13 +1136,14 @@ syntaxtree *trp;
 				if (!strcmp(&(s[1]), signallist[i].ident))
 					break;
 			if (signallist[i].sig < 0) {
-				execerror(&(s[1]), ER_UNKNOWNSIG, 0);
+				execerror((trp -> comm) -> argv,
+					&(s[1]), ER_UNKNOWNSIG, 1);
 				return(RET_FAIL);
 			}
 			sig = signallist[i].sig;
 		}
 		if (sig < 0 || sig >= NSIG) {
-			execerror(s, ER_NUMOUTRANGE, 0);
+			execerror((trp -> comm) -> argv, s, ER_NUMOUTRANGE, 1);
 			return(RET_FAIL);
 		}
 		i = 2;
@@ -1202,8 +1157,10 @@ syntaxtree *trp;
 #ifndef	NOJOB
 		if (*s == '%') {
 			if ((n = getjob(s)) < 0) {
-				execerror(s, (n < -1)
-					? ER_TERMINATED : ER_NOSUCHJOB, 0);
+				execerror((trp -> comm) -> argv,
+					s,
+					(n < -1)
+					? ER_TERMINATED : ER_NOSUCHJOB, 2);
 				return(RET_FAIL);
 			}
 			n = Xkillpg(joblist[n].pids[0], sig);
@@ -1211,16 +1168,13 @@ syntaxtree *trp;
 		else
 #endif	/* !NOJOB */
 		if ((pid = isnumeric(s)) < (p_id_t)0) {
-			fputs("usage: kill [ -sig ] pid ...\n", stderr);
-			fputs("for a list of signals: kill -l", stderr);
-			fputnl(stderr);
+			Xfputs("usage: kill [ -sig ] pid ...\n", Xstderr);
+			Xfputs("for a list of signals: kill -l", Xstderr);
+			VOID_C fputnl(Xstderr);
 			return(RET_SYNTAXERR);
 		}
 #if	MSDOS
-		else if (pid && pid != mypid) {
-			errno = EPERM;
-			n = -1;
-		}
+		else if (pid && pid != mypid) n = seterrno(EPERM);
 #endif
 		else n = kill(pid, sig);
 
@@ -1256,8 +1210,14 @@ int *ptrp;
 				? RET_SUCCESS : RET_FAIL;
 			break;
 		case 'f':
+#if	defined (BASHSTYLE) || defined (STRICTPOSIX)
+	/* "test" on bash & POSIX regards -f as checking if regular file. */
+			if (s) ret = (*s && Xstat(s, &st) >= 0
+			&& (st.st_mode & S_IFMT) == S_IFREG)
+#else
 			if (s) ret = (*s && Xstat(s, &st) >= 0
 			&& (st.st_mode & S_IFMT) != S_IFDIR)
+#endif
 				? RET_SUCCESS : RET_FAIL;
 			break;
 		case 'd':
@@ -1376,6 +1336,11 @@ int *ptrp;
 
 		if (s[0] == '!' && s[1] == '=' && !s[2])
 			ret = (strcmp(a1, a2)) ? RET_SUCCESS : RET_FAIL;
+#ifdef	BASHSTYLE
+	/* bash's "test" allows "==" as same as "=" */
+		else if (s[0] == '=' && s[1] == '=' && !s[2])
+			ret = (!strcmp(a1, a2)) ? RET_SUCCESS : RET_FAIL;
+#endif
 		else if (s[0] == '=' && !s[1])
 			ret = (!strcmp(a1, a2)) ? RET_SUCCESS : RET_FAIL;
 		else if (s[0] == '-') {
@@ -1473,8 +1438,8 @@ syntaxtree *trp;
 	if (argc <= 0) return(RET_FAIL);
 	if (argv[0][0] == '[' && !argv[0][1]
 	&& (--argc <= 0 || argv[argc][0] != ']' || argv[argc][1])) {
-		fputs("] missing", stderr);
-		fputnl(stderr);
+		Xfputs("] missing", Xstderr);
+		VOID_C fputnl(Xstderr);
 		return(RET_NOTICE);
 	}
 	if (argc <= 1) return(RET_FAIL);
@@ -1484,17 +1449,18 @@ syntaxtree *trp;
 	if (ret < 0) {
 		switch (ret) {
 			case -1:
-				fputs("argument expected", stderr);
+				Xfputs("argument expected", Xstderr);
 				break;
 			case -2:
-				fprintf2(stderr, "unknown operator %k",
+				VOID_C Xfprintf(Xstderr,
+					"unknown operator %k",
 					argv[ptr]);
 				break;
 			default:
-				fputs("syntax error", stderr);
+				Xfputs("syntax error", Xstderr);
 				break;
 		}
-		fputnl(stderr);
+		VOID_C fputnl(Xstderr);
 		return(RET_NOTICE);
 	}
 
@@ -1524,7 +1490,7 @@ syntaxtree *trp;
 # endif
 	if (flag == 'V') {
 		for (i = n; i < argc; i++) {
-			if (typeone(argv[i], stdout) >= 0) {
+			if (typeone(argv[i], Xstdout) >= 0) {
 # ifdef	BASHSTYLE
 				ret = RET_SUCCESS;
 # endif
@@ -1534,11 +1500,7 @@ syntaxtree *trp;
 	}
 	else if (flag == 'v') {
 		for (i = n; i < argc; i++) {
-#ifdef	NOALIAS
-			type = checktype(argv[i], &id, 0);
-#else
-			type = checktype(argv[i], &id, 0, 0);
-#endif
+			type = checktype2(argv[i], &id, 0, 0);
 
 			cp = argv[i];
 			if (type == CT_COMMAND) {
@@ -1550,14 +1512,14 @@ syntaxtree *trp;
 # endif
 				if ((type & CM_NOTFOUND)
 				|| Xaccess(cp, X_OK) < 0) {
-					execerror(cp, ER_COMNOFOUND, 0);
+					execerror(argv, cp, ER_COMNOFOUND, 1);
 					cp = NULL;
 				}
 			}
 
 			if (cp) {
-				kanjifputs(cp, stdout);
-				fputnl(stdout);
+				kanjifputs(cp, Xstdout);
+				VOID_C fputnl(Xstdout);
 # ifdef	BASHSTYLE
 				ret = RET_SUCCESS;
 # endif
@@ -1566,36 +1528,23 @@ syntaxtree *trp;
 		return(ret);
 	}
 
-#ifdef	NOALIAS
-	type = checktype(argv[n], &id, 0);
-#else
-	type = checktype(argv[n], &id, 0, 0);
-#endif
+	type = checktype2(argv[n], &id, 0, 0);
 	if (verboseexec) {
-		fprintf2(stderr, "+ %k", argv[n]);
+		VOID_C Xfprintf(Xstderr, "+ %k", argv[n]);
 		for (i = n + 1; i < argc; i++)
-			fprintf2(stderr, " %k", argv[i]);
-		fputnl(stderr);
+			VOID_C Xfprintf(Xstderr, " %k", argv[i]);
+		VOID_C fputnl(Xstderr);
 	}
 
 	(trp -> comm) -> argc -= n;
 	(trp -> comm) -> argv += n;
-	if (flag != 'p')
-#if	MSDOS
-		ret = exec_simplecom(trp, type, id);
-#else
-		ret = exec_simplecom(trp, type, id, 0);
-#endif
+	if (flag != 'p') ret = exec_simplecom2(trp, type, id, 0);
 	else {
-		path = strdup2(getconstvar(ENVPATH));
+		path = Xstrdup(getconstvar(ENVPATH));
 		setenv2(ENVPATH, DEFPATH, 1);
-#if	MSDOS
-		ret = exec_simplecom(trp, type, id);
-#else
-		ret = exec_simplecom(trp, type, id, 0);
-#endif
+		ret = exec_simplecom2(trp, type, id, 0);
 		setenv2(ENVPATH, path, 1);
-		if (path) free(path);
+		Xfree(path);
 	}
 	(trp -> comm) -> argc = argc;
 	(trp -> comm) -> argv = argv;
@@ -1614,11 +1563,11 @@ syntaxtree *trp;
 	argv = (trp -> comm) -> argv;
 
 	if (argc <= 2) {
-		execerror(NULL, ER_MISSARG, 0);
+		execerror(argv, NULL, ER_MISSARG, 1);
 		return(RET_FAIL);
 	}
 	if (identcheck(argv[2], '\0') <= 0) {
-		execerror(argv[2], ER_NOTIDENT, 0);
+		execerror(argv, argv[2], ER_NOTIDENT, 0);
 		return(RET_FAIL);
 	}
 
@@ -1666,12 +1615,13 @@ syntaxtree *trp;
 	if (ret != RET_SUCCESS) n = '?';
 	else {
 		n = argv[posixoptind][ptr++];
-		if (n == ':' || ismsb(n) || !(cp = strchr2(optstr, n))) {
+		if (n == ':' || ismsb(n) || !(cp = Xstrchr(optstr, n))) {
 			buf[0] = n;
 			buf[1] = '\0';
 			n = '?';
 			if (quiet) posixoptarg = buf;
-			else execerror(buf, ER_BADOPTIONS, 2);
+			else execerror((trp -> comm) -> argv,
+				buf, ER_BADOPTIONS, 2);
 		}
 		else if (*(cp + 1) == ':') {
 			if (argv[posixoptind][ptr])
@@ -1687,7 +1637,8 @@ syntaxtree *trp;
 				}
 				else {
 					n = '?';
-					execerror(buf, ER_MISSARG, 2);
+					execerror((trp -> comm) -> argv,
+						buf, ER_MISSARG, 2);
 				}
 			}
 			ptr = 0;
@@ -1702,11 +1653,11 @@ syntaxtree *trp;
 	setenv2(name, buf, 0);
 
 	n = posixoptind;
-	snprintf2(buf, sizeof(buf), "%d", n);
+	VOID_C Xsnprintf(buf, sizeof(buf), "%d", n);
 	setenv2(ENVOPTIND, buf, 0);
 	posixoptind = n;
 
 	return(ret);
 }
 #endif	/* NOPOSIXUTIL */
-#endif	/* !MINIMUMSHELL && (!FD || (FD >= 2 && !_NOORIGSHELL)) */
+#endif	/* DEP_ORIGSHELL && !MINIMUMSHELL */
